require 'tropy'
require 'koya'

class KoyaTropy < AbstractTropy
  class Leaf < Koya::KoyaObject
    def initialize(key, left = nil)
      self.key = key
      if left
        self.forward = left.forward
        left.forward = self
      else
        self.forward = self
      end
    end
    koya_attr :key, :text, :title, :body, :forward
    
    def to_hash
      {
        :key => self.key,
        :text => self.text,
        :title => self.title,
        :body => self.body
      }
    end
  end
  
  def initialize(db='tropy_koya.db')
    super()
    @koya = Koya::Store.new(db, false)
    @root = @koya.root
    @root.transaction do
      unless @root['dict']
        @root['dict'] = Koya::Dict.new
        @root['list'] = nil
        do_replace(0, "Tropy\na tropy clone.")
      end
    end
  end

  def add(text)
    replace(0, text)
  end
  
  def replace(key, text)
    key = make_key if key == 0
    do_replace(key, text)
  end

  def gc
    @koya.gc
  end

  def any
    @root.transaction do
      rotate(rand(7) + 3)
    end
  end
  
  def [](key)
    @root.transaction do
      (@root['dict'][key] || @root['dict'][0]).to_hash
    end
  end

  private
  def rotate(n)
    cur = @root['list']
    n.times do
      cur = cur.forward
    end
    @root['list'] = cur
    cur.to_hash
  end

  def do_replace(key, text)
    title, body = get_title_body(text)

    @root.transaction do
      leaf = @root['dict'][key]
      unless leaf
        leaf = Leaf.new(key, @root['list'])
        @root['dict'][key] = leaf
        @root['list'] = leaf
      end
      leaf.text = text
      leaf.title = title
      leaf.body = body
      leaf.to_hash
    end
  end
end

